! Copyright (C)  2007  P.-A. von Wolffersdorff
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
!  USA.

! Subroutines in this file:
! Exported:
!  Subroutine GetModelCount( nMod )
!  Subroutine GetModelName ( iMod , ModelName )
!  Subroutine GetParamCount( iMod , nParam )
!  Subroutine GetParamName ( iMod , iParam, ParamName )
!  Subroutine GetParamUnit ( iMod , iParam, Units )
!  Subroutine GetStateVarCount( iMod , nVar )
!  Subroutine GetStateVarName ( iMod , iVar, Name )
!  Subroutine GetStateVarUnit ( iMod , iVar, Unit )
!
! Local:
!  Subroutine GetParamAndUnit( iMod , iParam, ParamName, Units )
!  Subroutine GetStateVarNameAndUnit( iMod , iVar, Name, Unit )
!


      Subroutine GetModelCount(nMod)
      !
      ! Return the maximum model number (iMod) in this DLL
      !
      Implicit None
      Integer (Kind=4) nMod
#ifdef lahey
      Dll_Export GetModelCount
#else
!DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetModelCount
#endif

      nMod = 2 ! Maximum model number (iMod) in current DLL

      Return
      End ! GetModelCount

      Subroutine GetModelName( iMod , ModelName )
      !
      ! Return the name of the different models
      !
      Implicit None
      Integer, intent(in) :: iMod
      Character (Len= * ) ModelName
      Character (Len=255) tName
      Integer LT
#ifdef lahey
      DLL_Export GetModelName
#else
!DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetModelName
#endif

      Select Case (iMod)
        Case (1)
          tName = 'Hypoplas.-sand'
        Case (2)
          tName = 'Hypoplas.-clay'
        Case Default
          tName = 'not in DLL'
      End Select
      LT = Len_Trim(tName)
      ModelName= Char(lt) // tName(1:Lt)
      Return
      End ! GetModelName

      Subroutine GetParamCount( iMod , nParam )
      !
      ! Return the number of parameters of the different models
      !
      Implicit None
      Integer, intent (in ) :: iMod
      Integer, intent (out) :: nParam
#ifdef lahey
      DLL_Export GetParamCount
#else
!DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamCount
#endif
      Select Case (iMod)
        Case (1)
          nParam = 22
        Case (2)
          nParam = 29
        Case Default
          nParam = 0
      End Select
      Return
      End ! GetParamCount

      Subroutine GetParamName( iMod , iParam, ParamName )
      !
      ! Return the parameters name of the different models
      !
      Implicit None
      Integer, intent(in) :: iMod, iParam
      Character (Len=255) ParamName, Units
#ifdef lahey
      DLL_Export GetParamName
#else
!DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamName
#endif
      Call GetParamAndUnit(iMod,iParam,ParamName,Units)
      Return
      End

      Subroutine GetParamUnit( iMod , iParam, Units )
      !
      ! Return the units of the different parameters of the different models
      !
      Character (Len=255) ParamName, Units
#ifdef lahey
      DLL_Export GetParamUnit
#else
!DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamUnit
#endif
      Call GetParamAndUnit(iMod,iParam,ParamName,Units)
      Return
      End

      Subroutine GetParamAndUnit( iMod , iParam, ParamName, Units )
      !
      ! Return the parameters name and units of the different models
      !
      ! Units: use F for force unit
      !            L for length unit
      !            T for time unit
      !
      Implicit None
      Integer, intent(in) :: iMod, iParam
      Character (Len=255) ParamName, Units, tName
      Integer LT
      Select Case (iMod)
        Case (1)
          ! ModName = 'Hypoplasticity - sand'
          Select Case (iParam)
            Case (1)
              ParamName = '@j#_c#'
              Units     = 'deg'
            Case (2)
              ParamName = 'p_t'
              Units     = 'F/L^2#'
            Case (3)
              ParamName = 'h_s#'
              Units     = 'F/L^2#'
            Case (4)
              ParamName = 'n'
              Units     = '-'
            Case (5)
              ParamName = 'e_d0#'
              Units     = '-'
            Case (6)
              ParamName = 'e_c0#'
              Units     = '-'
            Case (7)
              ParamName = 'e_i0#'
              Units     = '-'
            Case (8)
              ParamName = '@a#'
              Units     = '-'
            Case (9)
              ParamName = '@b#'
              Units     = '-'
            Case (10)
              ParamName = 'm_R#'
              Units     = '-'
            Case (11)
              ParamName = 'm_T#'
              Units     = '-'
            Case (12)
              ParamName = 'R_max#'
              Units     = '-'
            Case (13)
              ParamName = '@b#_r#'
              Units     = '-'
            Case (14)
              ParamName = '@c#'
              Units     = '-'
            Case (15)
              ParamName = '-'
              Units     = '-'
            Case (16)
              ParamName = 'SV: e_0# or e'
              Units     = '-'
            Case (17)
              ParamName = 'SV: is0_11#'
              Units     = '-'
            Case (18)
              ParamName = 'SV: is0_22#'
              Units     = '-'
            Case (19)
              ParamName = 'SV: is0_33#'
              Units     = '-'
            Case (20)
              ParamName = 'SV: is0_12#'
              Units     = '-'
            Case (21)
              ParamName = 'SV: is0_13#'
              Units     = '-'
            Case (22)
              ParamName = 'SV: is0_23#'
              Units     = '-'
            Case Default
              ParamName = '???'
              Units     = '???'
          End Select
        Case (2)
          ! ModName = 'Hypoplasticity - clay'
          Select Case (iParam)
            Case (1)
              ParamName = '@j#_c#'
              Units     = 'deg'
            Case (2)
              ParamName = 'p_t'
              Units     = 'F/L^2#'
            Case (3)
              ParamName = '@l#*#'
              Units     = '-'
            Case (4)
              ParamName = '@k#*#'
              Units     = '-'
            Case (5)
              ParamName = 'N'
              Units     = '-'
            Case (6)
              ParamName = '@n_hh#'
              Units     = '-'
            Case (7)
              ParamName = '@a_G# (anisot)'
              Units     = '-'
            Case (8)
              ParamName = '( k )'
              Units     = '-'
            Case (9)
              ParamName = '( A )'
              Units     = '-'
            Case (10)
              ParamName = '( s_f# )'
              Units     = '-'
            Case (11)
              ParamName = 'R_max#'
              Units     = '-'
            Case (12)
              ParamName = '@b#_r#'
              Units     = '-'
            Case (13)
              ParamName = '@c#'
              Units     = '-'
            Case (14)
              ParamName = 'G_vh0#'
              Units     = '-'
            Case (15)
              ParamName = 'n'
              Units     = '-'
            Case (16)
              ParamName = 'm_T#/m_R# ratio'
              Units     = '-'
            Case (17)
              ParamName = 'K_w#'
              Units     = 'F/L^2#'
            Case (18)
              ParamName = 'vert. (1/2/3)'
              Units     = '-'
            Case (19)
              ParamName = '@a_E#'
              Units     = '-'
            Case (20)
              ParamName = '@a_@n#'
              Units     = '-'
            Case (21)
              ParamName = '@a_f#'
              Units     = '-'
            Case (22)
              ParamName = 'SV: e or OCR'
              Units     = '-'
            Case (23)
              ParamName = 'SV: is0_11#'
              Units     = '-'
            Case (24)
              ParamName = 'SV: is0_22#'
              Units     = '-'
            Case (25)
              ParamName = 'SV: is0_33#'
              Units     = '-'
            Case (26)
              ParamName = 'SV: is0_12#'
              Units     = '-'
            Case (27)
              ParamName = 'SV: is0_13#'
              Units     = '-'
            Case (28)
              ParamName = 'SV: is0_23#'
              Units     = '-'
            Case (29)
              ParamName = 'SV: sensit.'
              Units     = '-'
            Case Default
              ParamName = '???'
              Units     = '???'
          End Select
        Case Default
          ! model not in DLL
          ParamName = ' N/A '
          Units     = ' N/A '
      End Select
      tName    = ParamName
      LT       = Len_Trim(tName)
      ParamName= Char(lt) // tName(1:Lt)

      tName = Units
      LT    = Len_Trim(tName)
      Units = Char(lt) // tName(1:Lt)

      Return
      End ! GetParamAndUnit

      Subroutine GetStateVarCount( iMod , nVar )
      !
      ! Return the number of state variables of the different models
      !
      Implicit None
      Integer  iMod, nVar

      !DEC$ ATTRIBUTES DLLExport, StdCall, reference :: GetStateVarCount

      Call Get_StateVar_Count( iMod , nVar )

      End ! GetStateVarCount

      Subroutine GetStateVarName( iMod , iVar, Name )
      !
      ! Return the name of the different state variables
      ! of the different models
      !
      Implicit None
      Integer  iMod, iVar
      Character (Len=255) Name, Unit

      !DEC$ ATTRIBUTES DLLExport, StdCall, reference  :: GetStateVarName

      Call GetStateVarNameAndUnit( iMod , iVar, Name, Unit )
      Call Add_Str_Length( Name )

      End ! GetStateVarName

      Subroutine GetStateVarUnit( iMod , iVar, Unit )
      !
      ! Return the units of the different state variables of the different models
      !
      Implicit None
      Integer  iMod, iVar
      Character (Len=255) Name, Unit

      !DEC$ ATTRIBUTES DLLExport, StdCall, reference  :: GetStateVarUnit

      Call GetStateVarNameAndUnit( iMod , iVar, Name, Unit )
      Call Add_Str_Length( Unit )

      End ! GetStateVarUnit

      Subroutine Add_Str_Length( aString )
      Implicit None
      Character*(*) aString
      Character *255 tString
      Integer Lt              ! length of incoming string

      ! routine should add the length of the string as the first character

      tString = aString
      Lt      = Len_Trim(tString)
      aString = Char(Lt) // tString(1:Lt)

      End Subroutine Add_Str_Length


      Subroutine Get_StateVar_Count( iMod , nVar )
      !
      ! Return the number of state variables of the different models
      !

      Implicit None
      Integer, intent(in ) :: iMod
      Integer, intent(out) :: nVar
      Select Case (iMod)
      Case (1)
        nVar = 14
      Case (2)
        nVar = 16
      Case Default
        nVar = 0
      End Select

      Return
      End


      Subroutine GetStateVarNameAndUnit( iMod , iVar, ParamName, Units )
      !
      ! Return the name and unit of the different state variables of the different models
      !
      Implicit None
      Integer, intent(in) :: iMod, iVar
      Character (Len=255) ParamName, Units

      ParamName = '?'
      Units     = '-'
      Select Case (iMod)
      Case (1)
        Select Case (iVar)
          Case (1)
            ParamName = 'is_11#'
          Case (2)
            ParamName = 'is_22#'
          Case (3)
            ParamName = 'is_33#'
          Case (4)
            ParamName = 'is_12#'
          Case (5)
            ParamName = 'is_13#'
          Case (6)
            ParamName = 'is_23#'
          Case (7)
            ParamName = 'e'
          Case (8)
            ParamName = 'not used'
          Case (9)
             ParamName = 'p_eff#'   ; Units     = 'F/L^2#'
          Case (10)
            ParamName = 'n_eval#'
          Case (11)
            ParamName = '@j#_mob#' ; Units     = 'deg'
          Case (12)
            ParamName = '@r#'
          Case (13)
            ParamName = 'suggested_time#'
        Case Default
          ParamName='N/A'              ; Units = '?'
        End Select
      Case (2)
        Select Case (iVar)
          Case (1)
            ParamName = 'is_11#'
          Case (2)
            ParamName = 'is_22#'
          Case (3)
            ParamName = 'is_33#'
          Case (4)
            ParamName = 'is_12#'
          Case (5)
            ParamName = 'is_13#'
          Case (6)
            ParamName = 'is_23#'
          Case (7)
            ParamName = 'e'
          Case (8)
            ParamName = 'EPP'      ; Units     = 'F/L^2#'
          Case (9)
            ParamName = 'p_eff#'   ; Units     = 'F/L^2#'
          Case (10)
            ParamName = 'n_eval#'
          Case (11)
            ParamName = '@j#_mob#' ; Units     = 'deg'
          Case (12)
            ParamName = '@r#'
          Case (13)
            ParamName = 'suggested_time#'
          Case (14)
            ParamName = 'sensitivity s'
          Case (15)
            ParamName = 'OCR'
          Case (16)
            ParamName = 'iter problems'
        Case Default
            ParamName='N/A'              ; Units = '?'
        End Select
      Case Default
        ParamName='N/A'                  ; Units = '?'
      End Select

      Return
      End



